<!---
$Id: abstracttool.cfc 2966 2007-11-18 19:06:48Z barneyb $

Copyright 2007 Barney Boisvert (bboisvert@gmail.com).

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<cfcomponent output="false">

	<cffunction name="init" access="public" output="false" returntype="abstracttool">
		<cfargument name="migrationPackage" type="string" required="true"
			hint="The package migration classes (named 'migrationNNNNNN.cfc') reside.  Can either be an absolute package name, or a relative (from this package) package name." />
		<cfset variables.my.migrationPackage = migrationPackage />
		<cfreturn this />
	</cffunction>



	<cffunction name="ensureCurrent" access="public" output="false" returntype="void">
		<cfargument name="lockName" type="string" required="true"
			hint="A unique lock name to use for performing schema updates within.  '##application.applicationname##.schema_tool' is usually a safe bet." />
		<cflock name="#lockName#" type="exclusive" timeout="1" throwontimeout="true">
			<cfset ensureCurrentInternal() />
		</cflock>
	</cffunction>



	<cffunction name="ensureCurrentInternal" access="private" output="false" returntype="void">
		<cfset var state = getState(true) />
		<cfset var state2 = "" />
		<cfset var maxMajor = "" />
		<cfset var major = "" />
		<cfset var migration = "" />
		<!--- at this point, we know what version the schema is at --->
		<cfset maxMajor = getMaxMajorVersion() />
		<cfset minor = state.minorVersion />
		<cfloop from="#max(1, state.majorVersion)#" to="#maxMajor#" index="major">
			<cfset setMajorVersion(major) />
			<cfset migration = getMigrationForVersion(major) />
			<cfset migration.migrate(minor) />
			<cfset minor = 0 />
		</cfloop>
		<cfset state2 = getState() />
		<cfif state.majorVersion NEQ state2.majorVersion
			OR state.minorVersion NEQ state2.minorVersion>
			<cftrace text="Migrated from #state.majorVersion#.#state.minorVersion# to #state2.majorVersion#.#state2.minorVersion#" />
		</cfif>
	</cffunction>
	
	
	
	<cffunction name="getMigrationForVersion" access="private" output="false" returntype="abstractmigration">
		<cfargument name="major" type="numeric" required="true" />
		<cfset var root = "migration" & numberFormat(major, "000000") />
		<cfset var dir = getMigrationPackageDir() />
		<cfif fileExists("#dir#/#root#.sql")>
			<!--- it's SQL based --->
			<cfreturn createObject("component", "sqlbasedmigration").init(this, "#dir#/#root#.sql") />
		</cfif>
		<cfreturn createObject("component", "#my.migrationPackage#.#root#").init(this) />
	</cffunction>
	
	
	
	<cffunction name="getMigrationPackageDir" access="private" output="false" returntype="string">
		<cfreturn expandPath("/#replace(my.migrationPackage, '.', '/', 'all')#") />
	</cffunction>
	
	
	
	<cffunction name="getMaxMajorVersion" access="private" output="false" returntype="numeric">
		<cfset var scripts = "" />
		<cfset var dir = getMigrationPackageDir() />
		<cfset var migrations = "" />
		<cfif NOT directoryExists(dir)>
			<!--- not an absolute package, let's see if it's a relative package --->
			<cfset dir = getDirectoryFromPath(getCurrentTemplatePath()) & replace(my.migrationPackage, '.', '/', 'all') />
		</cfif>		
		<cfdirectory action="list"
			directory="#dir#"
			name="migrations"
			sort="name"
			filter="migration??????.???" />
		<cfquery dbtype="query" name="migrations">
			select name
			from migrations
			where name like '%.cfc'
				or name like '%.sql'
			order by name
		</cfquery>
		<cfif migrations.recordCount EQ 0>
			<cfreturn 0 />
		<cfelse>
			<cfreturn val(REReplace(migrations.name[migrations.recordCount], "[^0-9]+", "", "all")) />
		</cfif>
	</cffunction>
	
	
	
	<cffunction name="getState" access="public" output="false" returntype="struct">
		<cfargument name="initialize" type="boolean" default="false"
			hint="Whether to initialize the persistence holder if needed." />
		<cfthrow type="UnimplementedExcpetion"
			message="You must implement getState on abstracttool subclasses" />
	</cffunction>
	
	
	
	<cffunction name="setMajorVersion" access="private" output="false" returntype="void">
		<cfargument name="majorVersion" type="numeric" required="true" />
		<cfset var state = getState() />
		<cfif state.majorVersion NEQ majorVersion>
			<cfset state.majorVersion = majorVersion />
			<cfset updateState(state) />
		</cfif>
	</cffunction>

	
	
	<cffunction name="setMinorVersion" access="public" output="false" returntype="void">
		<cfargument name="minorVersion" type="numeric" required="true" />
		<cfset var state = getState() />
		<cfset state.minorVersion = minorVersion />
		<cfset updateState(state) />
	</cffunction>
	
	
	
	<cffunction name="updateState" access="private" output="false" returntype="void">
		<cfargument name="state" type="struct" required="true" />
		<cfthrow type="UnimplementedExcpetion"
			message="You must implement updateState on abstracttool subclasses" />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>