<!---
I'm the centerpoint of the package.  I should be initialized and then
have my ensureCurrent method invoked, which will take care of the rest.

I don't do any work, instead I rely on a collection of migration scripts
(sequentially named migrationNNNNNN.cfc, where N is a digit), each of
which represents a major version of the schema.  Each migration script
must have a 'migrate' method that accepts the schema's current minor
version and does any updates necessary to increase it's minor version as
high as it can go.  Migration scripts will be called in order, starting
with the current major version (in case additional minor versions have
been added), and progressing upwards through all defined migration
scripts.

The semantics of a minor version are entirely up to the individual
migration scripts, but a general rule of thumb is that each one should
be a single DDL statement or a single transaction (of DML).  An abstract
base class (abstractmigration.cfc) has been provided as a starting
point, along with two subclasses: switchbasedmigration.cfc and
sqlbasedmigration.cfc.  Consult their comments for instructions on how
to use them.

------------------------------------------------------------------------
$Id: databasetool.cfc 2966 2007-11-18 19:06:48Z barneyb $

Copyright 2007 Barney Boisvert (bboisvert@gmail.com).

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<cfcomponent output="false" extends="abstracttool">

	<cffunction name="init" access="public" output="false" returntype="databasetool">
		<cfargument name="migrationPackage" type="string" required="true"
			hint="The package migration classes (named 'migrationNNNNNN.cfc') reside.  Can either be an absolute package name, or a relative (from this package) package name." />
		<cfargument name="dsn" type="string" required="true" />
		<cfset super.init(migrationPackage) />
		<cfset variables.my.dsn = dsn />
		<cfreturn this />
	</cffunction>
	
	
	
	<cffunction name="getState" access="public" output="false" returntype="struct">
		<cfargument name="initialize" type="boolean" default="false"
			hint="Whether to initialize the persistence holder if needed." />
		<cfset var get = "" />
		<cfset var result = structNew() />
		
		<cftry>
			<cfquery datasource="#variables.my.dsn#" name="get">
				select major_version, minor_version
				from schema_version
			</cfquery>
			<cfset result["majorVersion"] = get.major_version />
			<cfset result["minorVersion"] = get.minor_version />
		
			<cfcatch type="database">
				<cfif initialize AND cfcatch.message & cfcatch.detail CONTAINS "schema_version">
					<cfquery datasource="#variables.my.dsn#" name="">
						create table schema_version (
							major_version int not null default 0,
							minor_version int not null default 0
						)
					</cfquery>
					<cfquery datasource="#variables.my.dsn#" name="">
						insert into schema_version
							(major_version, minor_version
							)
						values
							(0,
							 0
							)
					</cfquery>
					<cfreturn getState() />
				<cfelse>
					<cfrethrow />
				</cfif>
			</cfcatch>
		</cftry>
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="getMigrationForVersion" access="private" output="false" returntype="abstractmigration">
		<cfargument name="major" type="numeric" required="true" />
		<cfset var result = super.getMigrationForVersion(major) />
		<cfset result.setDsn(variables.my.dsn) />
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="updateState" access="private" output="false" returntype="void">
		<cfargument name="state" type="struct" required="true" />
		<cfquery datasource="#variables.my.dsn#" name="">
			update schema_version set
				major_version = <cfqueryparam cfsqltype="cf_sql_integer" value="#state.majorVersion#" />,
				minor_version = <cfqueryparam cfsqltype="cf_sql_integer" value="#state.minorVersion#" />
		</cfquery>
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>
