<!---
$Id: filesystemtool.cfc 2966 2007-11-18 19:06:48Z barneyb $

Copyright 2007 Barney Boisvert (bboisvert@gmail.com).

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<cfcomponent output="false" extends="abstracttool">

	<cffunction name="init" access="public" output="false" returntype="filesystemtool">
		<cfargument name="migrationPackage" type="string" required="true"
			hint="The package migration classes (named 'migrationNNNNNN.cfc') reside.  Can either be an absolute package name, or a relative (from this package) package name." />
		<cfargument name="filename" type="string" required="true"
			hint="The file to use for versioning.  Do NOT put this file into your source control repository, nor somewhere that it will be overwritten when you deploy code.  Somewhere inside '/WEB-INF/' is often a good place, otherwise somewhere outside your app root." />
		<cfset super.init(migrationPackage) />
		<cfset variables.my.filename = filename />
		<cfreturn this />
	</cffunction>
	
	
	
	<cffunction name="getState" access="public" output="false" returntype="struct">
		<cfargument name="initialize" type="boolean" default="false"
			hint="Whether to initialize the persistence holder if needed." />
		<cfset var fileContent = "" />
		<cfset var result = structNew() />
		
		<cftry>
			<cffile action="read"
				file="#my.filename#"
				variable="fileContent" />
			<cfset fileContent = trim(fileContent) />
			<cfset fileContent = listLast(fileContent, ":") />
			<cfset result["majorVersion"] = listFirst(fileContent) />
			<cfset result["minorVersion"] = listRest(fileContent) />
			
			<cfif NOT isNumeric(result["majorVersion"]) OR NOT isNumeric(result["minorVersion"])>
				<cfthrow type="InvalidStateException"
					message="The migration tool's state file is invalid." />
			</cfif>
			
			<cfcatch type="any">
				<cfif initialize>
					<cfset result = {
						majorVersion = 0,
						minorVersion = 0
					} />
					<cfset updateState(result) />
					<cfreturn getState() />
				<cfelse>
					<cfrethrow />
				</cfif>
			</cfcatch>
		</cftry>
		<cfreturn result />
	</cffunction>
	
	
	
	<cffunction name="updateState" access="public" output="false" returntype="void">
		<cfargument name="state" type="struct" required="true" />
		<cffile action="write"
			file="#my.filename#"
			output="schema_tool.filesystemtool:#state.majorVersion#,#state.minorVersion#" />
	</cffunction>

<cfoutput></cfoutput>
</cfcomponent>
