<!---
I am a utility class that makes simple migration script implementation
a bit easier.  I carry a few usage assumptions in order to work my
magic in it's entirety.  First and foremost, subclasses must implement
a doMinorVersion method that accepts an integer for the minor version
to upgrade.

Furthermore, the implementation of doMinorVersion must use a CFSWITCH
with an expression of "#minorVersion#" (the passed minor version).  It
must place it's CFDEFAULTCASE (if it has one) after all CFCASE tags, and
it may not have any non-numeric CFCASE value.  The CFCASEs may be in any
order (though it's recommended they are in ascending numerical order).

This class's getMaxMinorVersion method works by parsing the literal CFML
of it's subclass to compute the return value, and bases it's parsing on
these assumptions.

If that is unacceptable behaviour (for whatever reason), it's often best
to still subclass this CFC, but to override getMaxMinorVersion as well
as doMinorVersion, as the migrate and setMinorVersion method
implementations have no outlandish dependencies, and provide a bunch of
the boilerplate code to make things work.  There is not a hard
dependency that all migration scripts subclass this CFC, but it should
be considered a best practice.

------------------------------------------------------------------------
$Id: switchbasedmigration.cfc 2966 2007-11-18 19:06:48Z barneyb $

Copyright 2007 Barney Boisvert (bboisvert@gmail.com).

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<cfcomponent output="false" abstract="true" extends="abstractmigration">
	
	<cffunction name="getMaxMinorVersion" access="public" output="false" returntype="numeric">
		<cfset var fileContent = "" />
		<cfset var i = "" />
		<cfset var lines = "" />
		<cfset var cfcaseRe = ".*<cfcase.*value=(""|')([0-9]+)\1.*" />
		<cffile action="read"
			file="#getCurrentTemplatePath()#"
			variable="fileContent" />
		
		<!--- extract the doMinorVersion method --->
		<cfset i = REFindNoCase("name=(""|')doMinorVersion\1", fileContent) />
		<cfif i LTE 0>
			<cfthrow type="UnsupportedMigration"
				message="The current migration doesn't support the default getMaxMinorVersion implementation, please override."
				detail="No doMinorVersion method" />
		</cfif>
		<cfset fileContent = removeChars(fileContent, 1, i) />

		<cfset i = findNoCase("</cffunction", fileContent) />
		<cfset fileContent = left(fileContent, i) />
		
		<!--- extract the #minorVersion# cfswitch --->
		<cfset i = REFindNoCase("expression=(""|')##minorVersion##\1", fileContent) />
		<cfif i LTE 0>
			<cfthrow type="UnsupportedMigration"
				message="The current migration doesn't support the default getMaxMinorVersion implementation, please override."
				detail="no expression=""##minorVersion##""" />
		</cfif>
		<cfset fileContent = removeChars(fileContent, 1, i) />

		<cfset i = findNoCase("</cfswitch", fileContent) />
		<cfset fileContent = left(fileContent, i) />
		
		<!--- kill the cfdefaultcase and everything after --->
		<cfset fileContent = REReplaceNoCase(fileContent, "<cfdefaultcase.+", "", "all") />
		
		<!--- extract numeric case values --->
		<cfset lines = listToArray(fileContent, "#chr(10)##chr(13)#") />
		<cfloop from="#arrayLen(lines)#" to="1" index="i" step="-1">
			<cfif REFind(cfcaseRe, lines[i]) GT 0>
				<cfset lines[i] = REReplace(lines[i], cfcaseRe, "\2") />
			<cfelse>
				<cfset arrayDeleteAt(lines, i) />
			</cfif>
		</cfloop>
		
		<cfif arrayLen(lines) EQ 0>
			<cfthrow type="UnsupportedMigration"
				message="The current migration doesn't support the default getMaxMinorVersion implementation, please override."
				detail="No numeric case blocks" />
		</cfif>
		
		<!--- return the max value as the max minor version --->
		<cfreturn arrayMax(lines) />
	</cffunction>
	
<cfoutput></cfoutput>
</cfcomponent>